#include "baghira.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qdialog.h>
//#include <qsizegrip.h>
#include <qstatusbar.h>
#include <qgroupbox.h>

void LiquidStyle::polish( QPalette &pal ) {
	if ( inExitPolish )
		return ;
	// clear out all old colorized pixmaps
	int i;
	for ( i = 0; i < BITMAP_ITEMS; ++i ) {
		if ( pixmaps[ i ] ) {
			delete pixmaps[ i ];
			pixmaps[ i ] = NULL;
			}
		}
	// clear out all dicts
	btnDict.clear();
	btnShadowedDict.clear();
	bevelFillDict.clear();
	smallBevelFillDict.clear();
//	optionHandler->pixDict.clear();
	optionHandler->popupDict.clear();
	// reset brushes
	pagerHoverBrush = QBrush();
	pagerBrush = QBrush();

	/**
	 * Okay, here's the deal. We might need to set the application
	 * background brush with a pixmap stipple or a custom kicker color.
	 * It wouldn't be much of a problem but Qt calls this method for a lot
	 * of different palettes, not just the application's, so we really don't
	 * know what we are changing and if the QPalette parameter is actually
	 * the application's colors or something like a tooltip.
	 *
	 * This used to mean I'd check the timestamp of .qt/qtrc and reload the
	 * entire palette if it has changed. Now we just assume the QPalette
	 * parameter is the new application palette. Much more efficent :)
	 */

	if ( !isKicker && isPlain() )
		return ;

	bool newPalette = false;
	struct stat buffer;
	if ( stat( QFile::encodeName( QDir::homeDirPath() + "/.qt/qtrc" ), &buffer ) == 0 ) {
		unsigned int lastModTime = ( unsigned int ) buffer.st_mtime;
		if ( lastModTime > qtrcModificationTime ) {
			qtrcModificationTime = lastModTime;
			newPalette = true;
			}
		} else if ( !initialPaletteLoaded )  // Hack, should always have qtrc in KDE
		newPalette = true;
	initialPaletteLoaded = true;

	if ( !newPalette ) {
		pal = polishedPalette;
		return ;
		}
		
	pal.setColor( QColorGroup::Button, optionHandler->CustomButtonColor() );

	// only executed on first run or if qtrc has changed
	if ( isKicker ) {
		origPanelBrush = pal.brush( QPalette::Active,
		                            QColorGroup::Background );
		origPanelPalette = pal;
		QColor c( pal.active().button() );
		if ( optionHandler->usePanelCustomColor() &&
		        optionHandler->panelCustomColor().isValid() ){
			c = optionHandler->panelCustomColor();
		}

		pal.setColor( QColorGroup::Mid, c.dark( 110 ) );
		pal.setColor( QColorGroup::Dark, c.dark( 130 ) );
		pal.setColor( QColorGroup::Midlight, c.light( 110 ) );
		pal.setColor( QColorGroup::Light, c.light( 115 ) );
		if (optionHandler->BrushMe()){
			QPixmap brushed = optionHandler->TintBrush() ? *tintBrush(fetchImage( "brush-me" ), optionHandler->BrushTint()) : fetchImage( "brush-me" );
			QBrush brush( c, brushed );
			pal.setBrush( QColorGroup::Background, brush );
		}
		else if ( optionHandler->usePanelStipple() ) {
			QPixmap stipple( 64, 64 );
			stipple.fill( c.rgb() );
			QPainter p;
			p.begin( &stipple );
			p.setPen( c.dark( 100 + optionHandler->stippleContrast() ) );
			for ( i = 0; i < 63; i += 4 ) {
				p.drawLine( 0, i, 63, i );
				p.drawLine( 0, i+1, 63, i+1 );
				}
			p.end();
			QBrush brush( c, stipple );
			pal.setBrush( QColorGroup::Background, brush );
		} else
			pal.setBrush( QColorGroup::Background, c );
	}
	else if (optionHandler->BrushMe()){
			QColor c( pal.active().background() );
			QPixmap brushed = optionHandler->TintBrush() ? *tintBrush(fetchImage( "brush-me" ), optionHandler->BrushTint()) : fetchImage( "brush-me" );
			QBrush brush( c, brushed );
			pal.setBrush( QColorGroup::Background, brush );
	}
	else if ( !isPlain() ) {
		origPanelBrush = pal.brush( QPalette::Active,QColorGroup::Button );
 		QColor c( pal.active().background() );
 			QPixmap stipple( 64, 64 );
			stipple.fill( c.rgb() );
			QPainter p;
			p.begin( &stipple );
			p.setPen( c.dark( 100 + optionHandler->stippleContrast() ) );
			for ( i = 0; i < 63; i += 4 ) {
				p.drawLine( 0, i, 63, i );
				p.drawLine( 0, i+1, 63, i+1 );
			}
 			p.end();
			QBrush brush( c, stipple );
			pal.setBrush( QColorGroup::Background, brush );
	}

	/*
	if(!isPlain()){ // do for both Kicker and normal apps
	    QColor c(pal.active().highlight().light(110));
	    QPixmap *pix = smallBevelFillDict.find(c.rgb());
	    if(!pix){
	        pix = new QPixmap(*smallBevelFillPix);
	        adjustHSV(*pix, c);
	        smallBevelFillDict.insert(c.rgb(), pix);
	    }
	    QBrush brush(c, *pix);
	    pal.setBrush(QColorGroup::Highlight, brush);

		}*/
	polishedPalette = pal;
	optionHandler->reset();
	}

void LiquidStyle::unPolish( QApplication *app ) {

	QPalette pal( app->palette() );
	inExitPolish = true;
// 	btnDict.setAutoDelete(true);btnDict.clear();
// 	btnShadowedDict.setAutoDelete(true);btnShadowedDict.clear();
// 	buttonGlowDict.setAutoDelete(true);buttonGlowDict.clear();
// 	comboDict.setAutoDelete(true);comboDict.clear();
// 	comboShadowedDict.setAutoDelete(true);comboShadowedDict.clear();
// 	tabDict.setAutoDelete(true);tabDict.clear();
// 	inverseTabDict.setAutoDelete(true);inverseTabDict.clear();
// 	rectBtnDict.setAutoDelete(true);rectBtnDict.clear();
// 	progress2Dict.setAutoDelete(true);progress2Dict.clear();
// 	bevelFillDict.setAutoDelete(true);bevelFillDict.clear();
// 	smallBevelFillDict.setAutoDelete(true);smallBevelFillDict.clear();
// 	optionHandler->popupDict.setAutoDelete(true);optionHandler->popupDict.clear();
	if ( isKicker ) {
		// reset kicker
		QColor c = origPanelBrush.color();
		pal.setBrush( QColorGroup::Background, c );
		pal.setBrush( QColorGroup::Mid, c.dark( 110 ) );
		pal.setBrush( QColorGroup::Dark, c.dark( 130 ) );
		pal.setBrush( QColorGroup::Midlight, c.light( 110 ) );
		pal.setBrush( QColorGroup::Light, c.light( 115 ) );
//		app->blockSignals( TRUE );
		app->setPalette( pal );
//		app->blockSignals( FALSE );
		} else if ( !isPlain() ) {
		// reset any stipples
		if ( pal.brush( QPalette::Active, QColorGroup::Background ).pixmap() )
			pal.setBrush( QColorGroup::Background, pal.active().background() );
		if ( pal.brush( QPalette::Active, QColorGroup::Button ).pixmap() )
			pal.setBrush( QColorGroup::Button, pal.active().button() );
//		app->blockSignals( TRUE );
		app->setPalette( pal );
//		app->blockSignals( FALSE );
		}

	/*if(isKicker || !isPlain())
	    pal.setBrush(QColorGroup::Highlight, pal.active().highlight());
	*/
	inExitPolish = false;
	}

void LiquidStyle::polish( QWidget *w ) {

	bool tmpBool = false;

//	qWarning( "widget: %s", w->className() );
//	qWarning("%s",qApp->argv() [ 0 ]);
//	( qstrcmp( qApp->argv() [ 0 ], "kicker" ) == 0 )

	// align the stipples
//	if (!isPlain() && !w->inherits("QPopupMenu") && !(w->isTopLevel() && isKicker)) w->setBackgroundOrigin(QWidget::WindowOrigin);
// bad idea, as it leads to several errors on some widgets, e.g. in kghostview and slows down khtml, i think

	//groupbox shadow stuff - don't try to catch by "w->inherits('QGroupBox')" or so. THIS HANDLES GROUPBOX CHILDREN
	if ( optionHandler->drawGroupBoxShadow() ){ // do we want nice sunken groupboxes with shadow?
		QWidget * w2;
		w2 = w->parentWidget(true);
		if ( w2 != 0 ) { // this check is VERY important !!! - leave it, and QT will fail to display anything (e.g. kdm won't start anymore)
			QGroupBox::QGroupBox * w3 = ( QGroupBox::QGroupBox* ) w2;
			// was my Daddy a Groupbox?
			bool iAmGroup = w2->inherits( "QGroupBox" ) && ( w3->frameShape() == QFrame::GroupBoxPanel ) && !(w3->isFlat())/* && (w2->backgroundMode() == Qt::PaletteBackground )*/ && !( ( w->inherits( "QScrollView" ) ) || ( w->inherits( "QLineEdit" ) ) || ( w->inherits( "QComboBox" ) ));
			if ( iAmGroup ) { // if daddy is a groupbox...
				if ( isKicker ){ // kicker (and children) needs special handling again - otherwise the shadow pixmap won't be found and your GB will be completely black
//					QPixmap *shadow = new QPixmap( 64, 64 );
					this->groupShadow = new QPixmap( 64, 64 );
					QColor c;
					( optionHandler->MenuColorButton() ) ? c = QColor( optionHandler->buttonColor() ) : c = QColor( optionHandler->bgColor() );
					this->groupShadow->fill( ( c.dark( 100 + optionHandler->ShadowDarkness() ) ).rgb() );
					if ( optionHandler->useBgStipple() ) {
						QPainter p;
						p.begin( this->groupShadow );
						p.setPen( c.dark( 100 + optionHandler->ShadowDarkness() + optionHandler->stippleContrast() ) );
						for ( int i = 0; i < 63; i += 4 ) {
							p.drawLine( 0, i, 63, i );
							p.drawLine( 0, i+1, 63, i+1 );
						}
					}
					w->setPaletteBackgroundPixmap ( *(this->groupShadow) );
//					delete shadow; // cleanup my memory !
				}
				else{
					w->setPaletteBackgroundPixmap ( *groupShadow ); // ... make me shadowed, too!
				}
//			w->setBackgroundOrigin(QWidget::WindowOrigin);
			}
		}
	// cannot (!) return;, as the widget itself is unhadled...
	}
	
// 	// first test, if we are toplevel...
// 	if (optionHandler->BrushMe() && w->isTopLevel() && ( w->minimumWidth() != w-> maximumWidth()) && !(w->inherits("QPopupMenu") || w->inherits("QListBox") || w->inherits("QLabel") || w->inherits("AlphaWidget") || w->inherits("ScrollWidget") )){
// 		qWarning("toplevel widget found: %s",w->className());
// 		w->installEventFilter( this );
// 	}

//	if (w->inherits( "KSqueezedTextLabel" )) qWarning("squeezelabel1");

// check this area!!! ==========================================================================000

	if ( w->inherits( "QButton" ) || w->inherits( "QComboBox" )) {
		if (w->colorGroup().button() == qApp->palette().active().background()){
			QPalette pal( w->palette() );
			pal.setBrush( QColorGroup::Button, qApp->palette().active().brush( QColorGroup::Button ) );
			w->setPalette( pal );
		}
		w->setBackgroundOrigin(QWidget::WindowOrigin);
//		w->setBackgroundMode( QWidget::PaletteBackground );
//		return ;
	}

	if (w->inherits( "QPushButton" )){
		w->setBackgroundMode( QWidget::PaletteBackground);
//		w->setBackgroundOrigin(QWidget::WindowOrigin);
		w->installEventFilter( this );
//		return;
	}

	if (w->inherits( "QComboBox" )){
		w->setBackgroundMode(NoBackground);
		w->installEventFilter( this );
//		return;
	}
	

	bool isViewport = qstrcmp( w->name(), "qt_viewport" ) == 0 || qstrcmp( w->name(), "qt_clipped_viewport" ) == 0;
	bool isViewportChild = w->parent() && ( ( qstrcmp( w->parent() ->name(), "qt_viewport" ) == 0 ) || ( qstrcmp( w->parent() ->name(), "qt_clipped_viewport" ) == 0 ) );
	
	if (w->inherits("TaskBar")){
		w->setBackgroundOrigin(QWidget::WidgetOrigin);
		goto kstpolish;
	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state1");

	if ( isViewportChild && w->inherits( "QRadioButton" ) && isHTMLWidget( w )) {
	// shape radiobuttons on html pages...
		w->setAutoMask( true );
		w->setBackgroundMode( QWidget::NoBackground );
		goto kstpolish;
	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state2");
	
	if (w->inherits("KDockWidgetAbstractHeaderDrag")){
		w->installEventFilter( this );
		goto kstpolish;
	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state3");
	
	if (w->inherits("QSpinWidget")){
		w->installEventFilter( this );
		goto kstpolish;
	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state4");
	
	if ( !isPlain () && (w->inherits( "KonqIconViewWidget" ) || w->inherits( "khtml"))) {
	// if you want custom backgrounds in konqueror, you have to reset pixmap, since Konq animation can't handle it
	// original liquid did this on khtml, too (at speed loss) so well see, if we get errors from this...
		QPalette pal( w->palette() );
		pal.setBrush( QColorGroup::Background, pal.active().background() );
		w->setPalette( pal );
	}
	
// 	if (w->inherits("QListView")){
// 		if (((QListView*)w)->header()->width() < w->width()){
// //			qWarning("expanding header");
// 			//((QListView*)w)->setResizeMode(QListView::LastColumn);
// 			((QListView*)w)->header()->setMinimumWidth(w->width());
// // 		qWarning("resize mode is %d", ((QListView*)w)->resizeMode ());
// // 		qWarning("header is %s stretched", ((QListView*)w)->header()->isStretchEnabled () ? "" : "not");
// 		}
// 		w->setMouseTracking( true );
// 		w->installEventFilter( this );
// 		goto kstpolish;
// 	}
	
	if ((w->inherits( "QScrollView" ) || w->inherits("KateView") || w->inherits("TEWidget") || w->inherits("CHexViewWidget")) && !(w->isA("KOAgenda"))){
		w->setMouseTracking( true );
		w->installEventFilter( this );
		goto kstpolish;
	}
	
//  	if (w->inherits("QDialog") && (w->minimumSize() != w->maximumSize())) {
//  		((QDialog*)w)->setSizeGripEnabled(true);
//  		goto kstpolish;
//  	}
 	else if (w->inherits("QStatusBar")/* && (w->minimumSize() != w->maximumSize())*/) {
//		((QStatusBar*)w)->setSizeGripEnabled(true);
 		w->installEventFilter( this );
 		goto kstpolish;
 	}
// 	else if (w->isTopLevel() && (w->minimumSize() != w->maximumSize())){
// 		QSizeGrip *grip = new QSizeGrip(w);
// 		goto kstpolish;
// 	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state5");
	
	if ( w->isA( "AppletHandleDrag" ) || w->inherits( "KMiniPagerButton" ) || w->inherits( "QSplitterHandle" ) || w->inherits( "TaskContainer" ) || w->inherits( "QLineEdit" ) ) {
//		w->setBackgroundOrigin(QWidget::WindowOrigin);
		w->installEventFilter( this );
		goto kstpolish;
	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state6");

	if ( !isPlain() ) {
		if ( !isViewport && w->parent() && qstrcmp( w->parent() ->name(), "proxyview" ) == 0 ) {
			w->setBackgroundOrigin( QWidget::WindowOrigin );
			goto kstpolish ;
		}
		
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state7");

		if ( w->ownPalette() && !w->inherits( "QButton" ) && !w->inherits( "QComboBox" ) )
			goto kstpolish ;
			
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state8");

		if ( w->inherits( "PanelButtonBase" ) )
			goto kstpolish ;
		
			
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state9");

		if ( !isKicker&& !isViewport && !isViewportChild && !w->testWFlags( WType_Popup ) && !w->inherits( "KDesktop" ) && !w->inherits( "PasswordDlg" ) ) {
			if ( w->backgroundMode() == QWidget::PaletteBackground || w->backgroundMode() == QWidget::PaletteButton ) {
				w->setBackgroundOrigin( QWidget::WindowOrigin );
				}
			}
		}

	if ( w->inherits( "QPushButton" ) || w->inherits( "QComboBox" )) goto kstpolish;
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state10");
	
// we should improve this stuff!!! ======================================================

	if ( w->inherits( "QMenuBar" ) ) {
		w->installEventFilter( this );
		w->setBackgroundMode( QWidget::PaletteBackground );
		w->setBackgroundOrigin(QWidget::WindowOrigin);
		goto kstpolish ;
	}
	
	
	if (w->inherits("QFrame")){
		w->setBackgroundOrigin(QWidget::WindowOrigin);
		if (((QFrame *)w)->frameShape() >= QFrame::Box && ((QFrame *)w)->frameShape() <= QFrame::WinPanel){
			qWarning("forcing down frameShape");
			((QFrame *)w)->setFrameShape(QFrame::StyledPanel);
		}
	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state11");

	if ( w->inherits( "QPopupMenu" )){
		optionHandler->prepareMenus();
		w->installEventFilter( optionHandler);
		w->setBackgroundMode( NoBackground );
		goto kstpolish ;
	}
	

	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state12");
	
	 if ( w->parentWidget() && ( ( w->inherits( "QListBox" ) && w->parentWidget()->inherits( "QComboBox" ) ) || w->inherits( "KCompletionBox" ) ) ) {
	// stolen from thin keramik... ;-)
		QListBox* listbox = (QListBox*) w;
		listbox->setBackgroundMode( NoBackground );
		w->installEventFilter( this );
		goto kstpolish;
	}

	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state13");
	
	if (w->inherits("QHeader")){
		w->setBackgroundMode( NoBackground );
		w->installEventFilter( this );
		goto kstpolish;
	}

	
//  	if (w->inherits("QTab")){
//  		qWarning("This is a Tab!");
//  		w->setMouseTracking( true );
//  		w->installEventFilter( this );
//  		goto kstpolish;
// 	}
	
//	if (w->inherits("QTabBar")){
//		w->setBackgroundOrigin(QWidget::WindowOrigin);
//		goto kstpolish;
//	}
	
// 	if (w->inherits("QSlider")){
// 		w->setBackgroundOrigin(QWidget::ParentOrigin);
// 		return;
// 	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state14");

	if (w->inherits("QProgressBar"))
	{
		w->setBackgroundMode( NoBackground );
		WidgetAnimator* progressAnimator = new WidgetAnimator(w, 2);
//		connect(timer, SIGNAL(timeout()), progressAnimator, SLOT(animateSingleWidget()));
//		connect(progressAnimator, SIGNAL(animate(bool)), w, SLOT(repaint(bool)));	
//		connect(timer, SIGNAL(timeout()), progressAnimator, SLOT(animateProgressBar()));
//		connect(progressAnimator, SIGNAL(animate()), w, SLOT(update()));
		goto kstpolish;
	}
	
	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state15");

// //----------------------------------------------------------------------------------------------------------------------------------------------------------------------
// /* OK, the following Part is a bit tricky!
// QT uses the Button Color to draw seperators on Toolbars.
// Unfortunately we cannot change this directly, so we have to set the button color of Toolbar Widgets to the background color.
// Unhappy Effect: Toolbuttons will lose the ability to highlight, as their Buttoncolor now changed to Background mode. (f***!)
// Solution: We reset the the Buttoncolor of Toolbuttons to Buttoncolor right after that.
// Unhappy Effect: Toolbuttons loose the ability to use redefined colors.
// Solution: Check, if button uses backgroud for color: then reset it to button
// Unhappy effect: An application cannot be redefine a Toolbutton color to background color.
// (The alternative would be to exclude apps like korn, that are known to redefine toolbutton color, but i think, this is the MUCH better way!)
// */
// 
// // 	if ( qstrcmp( w->name(), "kde toolbar widget" ) == 0 ) {
// // 		// can't just set the background mode to background because the widget
// // 		// resets it to button :P
// // //		qWarning("hello ktw");
// // 		QPalette pal( w->palette() );
// // 		pal.setBrush( QColorGroup::Button, qApp->palette().active().brush( QColorGroup::Background ) );
// // 		w->setBackgroundOrigin(QWidget::ParentOrigin);
// // 		w->setPalette( pal );
// // 		return ;
// // 	}
// 
//reset the toolbar button color to background
	if ( (w->inherits("QToolBar"))) {
		if (w->inherits("KToolBar")) ((QToolBar*)w)->boxLayout()->setSpacing( 0 );
		((QToolBar*)w)->setMargin( 4 );
		// can't just set the background mode to background because the widget
		// resets it to button :P
		QPalette pal( w->palette() );
		pal.setBrush( QColorGroup::Button, qApp->palette().active().brush( QColorGroup::Background ) );
		w->setBackgroundOrigin(QWidget::WindowOrigin);
		w->setPalette( pal );
		goto kstpolish ;
	}

 	if (tmpBool = w->inherits( "QToolButton" ) ||  w->inherits( "QToolBarSeparator" ) ||  w->inherits( "KToolBarSeparator" )){
// 		if (w->colorGroup().button() == qApp->palette().active().background()){
// 			QPalette pal( w->palette() );
// 			pal.setBrush( QColorGroup::Button, qApp->palette().active().brush( QColorGroup::Button ) );
// 			w->setPalette( pal );
// 		}
		if (tmpBool && w->parentWidget() && w->parentWidget()->inherits("QToolBar")){
			w->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
		}
 		w->setBackgroundMode( QWidget::PaletteBackground );
 		w->setBackgroundOrigin(QWidget::WindowOrigin);
// //		w->setBackgroundMode( NoBackground);
 		w->installEventFilter( this );
 		goto kstpolish;
 	}
// 	
// 	//if (w->inherits( "KonqIconViewWidget" )) qWarning("state17");
// //----------------------------------------------------------------------------------------------------------------------------------------------------------------------


// 	if (w->inherits( "KHTMLView" )){
// 		w->setBackgroundMode( NoBackground );
// 		return;
// 	}


	if ( !isPlain() ) {
		if ( w->inherits( "KActiveLabel" ) || (w->inherits( "QLabel" ) && !(w->inherits( "QTipLabel" ) || w->inherits( "KSystemTray" )))) {
//			if (w->inherits( "KSqueezedTextLabel" )) qWarning("squeezelabel2");
			w->setBackgroundMode( QWidget::PaletteBackground );
			w->setBackgroundOrigin(QWidget::WindowOrigin);
			w->installEventFilter( this );
			goto kstpolish ;
			}
		else if ( w->inherits( "QTipLabel" ) ) {
			w->setPalette( tooltipPalette );
			w->installEventFilter( this );
			goto kstpolish ;
			}
		}

	if ( w->inherits( "QDockWindow" ) ) {
		w->setBackgroundMode( QWidget::PaletteBackground );
		w->setBackgroundOrigin(QWidget::WindowOrigin);
		w->installEventFilter( this );
		goto kstpolish ;
	}

	if (w->inherits( "QRadioButton" ) || w->inherits( "QCheckBox" )){
		w->setBackgroundOrigin(QWidget::WindowOrigin);
		w->installEventFilter( this );
		goto kstpolish;
	}


	if ( w->inherits( "QScrollBar" ) ) {
		w->setCursor(Qt::PointingHandCursor);
		w->installEventFilter( this );
//		WidgetAnimator *tmpAnim = animatedWidgetDict[w->name()];
// 		if (!tmpAnim){
// 			WidgetAnimator* scrollbarAnimator = new WidgetAnimator(w, 0);
// 			scrollbarAnimator->setActiveWidget((QWidget **)(&activeScroller));
// 			animatedWidgetDict.insert(w->name(), scrollbarAnimator);
// 		}
//		else{
//			tmpAnim->setMyWidget(w);
//		}
		w->setBackgroundMode( QWidget::NoBackground );
		goto kstpolish ;
	}

/*	if ( w->inherits( "QHeader" ) ) {
		w->setMouseTracking( true );
		w->installEventFilter( this );
	}*/

	if ( w->isTopLevel() && isKicker ) {
	// more kicker stuff
		// force extensions and child panels to use proper palette.
		if ( !w->inherits( "Panel" ) && !w->inherits( "KPanelApplet" ) ) {
			if ( !isPlain() && !origPanelPalette.active().brush( QColorGroup::Background ).pixmap() ) {
				QBrush brush( origPanelPalette.active().brush( QColorGroup::Background ) );
				QColor c( brush.color() );
				QPixmap stipple( 64, 64 );
				stipple.fill( c.rgb() );
				QPainter p;
				p.begin( &stipple );
				p.setPen( c.dark( 100 + optionHandler->stippleContrast() ) );
				for ( int i = 0; i < 63; i += 4 ) {
					p.drawLine( 0, i, 63, i );
					p.drawLine( 0, i+1, 63, i+1 );
				}
				p.end();
				brush.setPixmap( stipple );
				origPanelPalette.setBrush( QColorGroup::Background, brush );
			}
			w->setPalette( origPanelPalette );
		}
		goto kstpolish ;
	}
kstpolish:	KStyle::polish( w ); // nothing of all above? do default kde stuff
	}



void LiquidStyle::unPolish( QWidget *w ) {

	bool isViewport;
	bool isViewportChild;

	if ( w->inherits("QToolBar") || qstrcmp( w->name(), "kde toolbar widget" ) == 0 ) {
		w->setBackgroundOrigin( QWidget::WidgetOrigin );
		w->unsetPalette();
		goto kstunpolish;
		}
	if ( w->inherits( "QPopupMenu" ) ) {
		w->unsetPalette();
		w->setBackgroundMode( QWidget::PaletteButton );
		w->removeEventFilter( optionHandler );
		goto kstunpolish;
		}
	if ( !isPlain () && w->inherits( "KonqIconViewWidget" ) ||
	        w->inherits( "KHTMLView" ) ) {
		w->unsetPalette();
		goto kstunpolish;
		}
	if (w->inherits("KateView") || w->inherits("TEWidget") || w->inherits("CHexViewWidget")){
//		w->setMouseTracking( false );
		w->removeEventFilter( this );
		goto kstunpolish;
	}

	if (w->isA( "QViewportWidget" ) || w->inherits( "QClipperWidget" ) )
		goto kstunpolish;

	if ( !isPlain() ) {
		if ( w->inherits( "KActiveLabel" ) || w->inherits( "QTipLabel" ) ) {
			w->unsetPalette();
			w->removeEventFilter( this );
			goto kstunpolish;
			}
		/*
		else if(w->inherits("KdmClock")){
		    ; // Check this!
		    return;
			}*/
		}
	else if ( w->inherits( "QMenuBar" ) ) {
		w->setBackgroundMode( QWidget::PaletteButton );
		w->removeEventFilter( this );
		if ( !isPlain() )
			w->setBackgroundOrigin( QWidget::WidgetOrigin );
		goto kstunpolish;
		}else if ( w->inherits( "QDockWindow" ) ) {
		w->setBackgroundMode( QWidget::PaletteButton );
		if ( !isPlain() )
			w->setBackgroundOrigin( QWidget::WidgetOrigin );
		w->removeEventFilter( this );
		goto kstunpolish;
		} else if ( w->inherits( "QComboBox" ) || w->inherits( "QPushButton" ) ) {
		w->setBackgroundMode( QWidget::PaletteButton );
		w->removeEventFilter( this );
		} else if ( w->inherits( "QRadioButton" ) ||
	            w->inherits( "QCheckBox" ) || w->inherits( "QScrollView" ) || w->isA( "AppletHandleDrag" ) ||
	            w->inherits( "KMiniPagerButton" ) || w->inherits( "QSplitterHandle" ) ||
	            w->inherits( "TaskContainer" ) || w->inherits( "QLineEdit" ) || w->inherits( "QListBox" ) || w->inherits( "QLabel" ) || w->inherits( "KCompletionBox" ) || w->inherits("KDockWidgetAbstractHeaderDrag") || w->inherits("QSpinWidget")) {
		} else if ( w->inherits( "QScrollBar" ) ) {
//		if (!w->isEnabled()) w->setEnabled(true);
		w->setBackgroundMode( QWidget::PaletteBackground );
		w->removeEventFilter( this );
		goto kstunpolish;
		} else if ( w->inherits( "QHeader" ) ) {
		w->setMouseTracking( false );
		w->removeEventFilter( this );
		}

	if ( w->inherits( "QToolButton" ) ) {
		w->removeEventFilter( this );
		w->unsetPalette();
		w->setBackgroundMode( QWidget::PaletteButton );
		if ( !isPlain() )
			w->setBackgroundOrigin( QWidget::WidgetOrigin );
		goto kstunpolish;
		}

	if ( w->isTopLevel() && isKicker && !w->inherits( "Panel" ) &&
	        !w->inherits( "KPanelApplet" ) ) {
		w->unsetPalette();
		goto kstunpolish;
		}

	isViewport = qstrcmp( w->name(), "qt_viewport" ) == 0 ||
	                  qstrcmp( w->name(), "qt_clipped_viewport" ) == 0;
	isViewportChild = w->parent() &&
	                       ( ( qstrcmp( w->parent() ->name(), "qt_viewport" ) == 0 ) ||
	                         ( qstrcmp( w->parent() ->name(), "qt_clipped_viewport" ) == 0 ) );

	if ( isViewportChild ) {
		if ( w->inherits( "QRadioButton" ) ) {
			if ( isHTMLWidget( w ) ) {
				w->setAutoMask( false );
				w->setBackgroundMode( QWidget::PaletteBackground );
				goto kstunpolish;
				}
			}
		}

	if ( !isPlain() ) {
		if ( !isViewport && w->parent() && qstrcmp( w->parent() ->name(), "proxyview" ) == 0 ) {
			w->setBackgroundOrigin( QWidget::WidgetOrigin );
			goto kstunpolish;
			}
		if ( w->ownPalette() && !w->inherits( "QButton" ) && !w->inherits( "QComboBox" ) )
			goto kstunpolish;
		if ( w->inherits( "PanelButtonBase" ) )
			goto kstunpolish;

		if ( !isViewport && !isViewportChild && !w->testWFlags( WType_Popup ) &&
		        !w->inherits( "KDesktop" ) && !w->inherits( "PasswordDlg" ) ) {

			if ( w->backgroundOrigin() == QWidget::WindowOrigin )
				w->setBackgroundOrigin( QWidget::WidgetOrigin );
			goto kstunpolish;
			}
		}
kstunpolish:	KStyle::unPolish( w );
	}
